'' =================================================================================================
''
''   File....... jm_read_text.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2011 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000

  BUF_SIZE = 80


con

  RX1    = 31
  TX1    = 30
  SDA    = 29
  SCL    = 28

  SD_CS  =  3 
  SD_DI  =  2
  SD_CLK =  1
  SD_DO  =  0


con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term : "fullduplexserial64"
  sd   : "fsrw" 


var

  byte  buf[BUF_SIZE+1]                                         ' string buffer

 
pub main | check, line

  term.start(RX1, TX1, %0000, 115_200)                          ' start serial output
  pause(1)

  term.str(string(CLS, "Text Reader", CR, CR))

  check := \sd.mount_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)      ' look for SD card
  if (check == 0)                                               '  if good, check is 0
    term.str(string("-- sd card mounted", CR))
    
  else
    term.str(string("-- sd card failed to mount", CR))

  if (check == 0)                                               ' if SD moutned
    check := \sd.popen(string("master.csv"), "r")               '  try to open master.csv (read)
    if (check == 0)                                             '  if file opened
      term.str(string("-- file opened", CR, CR))                '   report
      line := 0                                                 '   initialize line count
      repeat                                                    '   loop until end of file
        check := readln(@buf)                                   '    read 1 line
        if (check > 0)                                          '    if line available
          term.rjdec(++line, 4)                                 '     update count and display
          term.tx(" ")
          term.tx(" ")
          term.str(@buf)                                        '     print line
          term.rjdec(bin2dec(@buf, 32), 13)                     '     convert line to decimal, display
          term.tx(CR)                                           '     end terminal output with CR
      
    else
      term.str(string("-- could not open file", CR))

  \sd.pclose                                                    ' close file
  \sd.unmount                                                   ' umount SD card
  
  repeat
    waitcnt(0)


pub readln(pntr) | c, len

'' reads line of text from open file
'' -- terminated by CR or EOF
'' -- removes linefeeds if present

  len  := 0                                                     ' index into string

  repeat
    c := \sd.pgetc                                              ' get a character
    if (c < 0)                                                  ' end of file
      if (len == 0)                                             ' if nothing
        return -1                                               ' return empty
      else
        quit
    if (c == CR)                                                ' if CR we're done with line
      quit
    else
      if (c <> LF)                                              ' if not a line feed
        byte[pntr++] := c                                       ' else move c to buffer
        len++                                                   ' update character count

  byte[pntr] := 0                                               ' terminate end of line
  return len


pub bin2dec(pntr, len) | bval, c

'' Converts binary string at pntr to decimal value
'' -- removes non-binary characters at head of string
'' -- allows underscore character in binary string
'' -- pntr is address of string buffer to convert
'' -- len is maximum number of binary characters to process

  bval := 0

  repeat                                                        ' remove non-binary header   
    c := byte[pntr]
    case c
      "0".."1":                                                 ' if binary character
        quit                                                    '  we're done
        
      other:
        pntr++                                                  ' else move pointer

  repeat while (len > 0)                                        ' convert string    
    c := byte[pntr]                                             ' get a character
    case c
      "0".."1":                                                 ' if binary
        bval := (bval << 1) | (c - "0")                         '  add to value
        pntr++                                                  '  advance pointer
        len--                                                   '  update chars to process

      "_":                                                      ' if underscore
        pntr++                                                  '  ignore; advance pointer

      other:                                                    ' if other character
        quit                                                    '  we're done

  return bval
    

pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}